@@ -28,6 +28,17 @@ PROXY_BIN = path.join(PROXY_FOLDER, PROXY_NAME)
 
 
 class IngeniumAPI:
+    # Onload is a class static
+    _onload = None
+
+    @property
+    def onload(self) -> Optional[Callable[["IngeniumAPI"], None]]:
+        return IngeniumAPI._onload
+
+    @onload.setter
+    def onload(self, val: Callable[["IngeniumAPI"], None]):
+        IngeniumAPI._onload = val
+
     _user: Optional[str]
     _pass: Optional[str]
     _host: Optional[str]
@@ -52,8 +63,11 @@ class IngeniumAPI:
         self._pass = None
         return self
 
-    async def load(self, just_login=False, hass_integration=False):
+    async def load(self, just_login=False, hass_integration=False, debug=False):
         my_env = environ.copy()
+        my_env["ENABLE_TRAFFICSERVER"] = str(self._user == "CalidadAire").lower()
+        if debug:
+            my_env["LOG_LEVEL"] = "debug"
         self._proxy = subprocess.Popen([PROXY_BIN], env=my_env, universal_newlines=True)
         await asyncio.sleep(1)
 
@@ -89,11 +103,15 @@ class IngeniumAPI:
                 self._objects.append(IngMeterBus(self, self._is_knx, address, ctype, components))
             elif ctype.is_tsif():
                 self._objects.append(IngSif(self, self._is_knx, address, ctype, components))
+            elif ctype.is_air_sensor():
+                self._objects.append(IngAirSensor(self, self._is_knx, address, ctype, components))
             elif ctype.is_busing_regulator():
                 self._objects.append(IngBusingRegulator(self, self._is_knx, address, ctype, components))
             elif ctype.is_thermostat():
                 self._objects.append(IngThermostat(self, self._is_knx, address, ctype, components))
 
+        if self.onload is not None:
+            self.onload()
         return True
 
     @property
@@ -162,6 +180,15 @@ class IngeniumAPI:
                         entities.append((obj, comp, i))
         return entities
 
+    def get_air_sensors(self):
+        entities = []
+        for obj in self.objects:
+            if isinstance(obj, IngAirSensor):
+                for comp in obj.components:
+                    entities.append((obj, comp, 0))
+                    entities.append((obj, comp, 1))
+        return entities
+
     def get_lights(self):
         entities = []
         for obj in self.objects:
