@@ -1,12 +1,11 @@
 import asyncio
 from typing import Optional, TYPE_CHECKING
 
-from .objects import Package, IngActuator, ACTUATOR_BLIND, IngMeterBus, IngSif, IngThermostat
+from .objects import Package, IngActuator, ACTUATOR_BLIND, IngMeterBus, IngSif, IngThermostat, IngAirSensor
 
 if TYPE_CHECKING:
     from .api import IngeniumAPI
 
-import json
 import aiohttp
 
 
@@ -41,8 +40,8 @@ class CustomConnection:
                 for c in o.components:
                     try:
                         c.update_notify()
-                    except RuntimeError:
-                        print("Error notifying of change!")
+                    except RuntimeError as e:
+                        print("Error notifying of change! " + str(e))
 
     async def _notify_state(self, address: int, available: bool):
         for o in self.api.objects:
@@ -51,8 +50,8 @@ class CustomConnection:
                 for c in o.components:
                     try:
                         c.update_notify()
-                    except RuntimeError:
-                        print("Error notifying of change!")
+                    except RuntimeError as e:
+                        print("Error notifying of change! " + str(e))
 
     async def async_connect(self, reconnect=False, just_login=False, hass_integration=False):
         self._hass_integration = hass_integration
@@ -93,7 +92,7 @@ class CustomConnection:
                 else:
                     print("Invalid package kind: " + str(res))
 
-            except aiohttp.ClientError as e:
+            except (aiohttp.ClientError, TypeError) as e:
                 print("Connection has been closed: " + str(e))
                 if not self._is_closed:
                     self._is_closed = True
@@ -138,6 +137,9 @@ async def initial_read(api: 'IngeniumAPI'):
             await api.connection.send(Package(0xFFFF, o.address, 10, 0, 0))
             # This is weird on the sif
             # await o.read_state()
+        elif isinstance(o, IngAirSensor):
+            # A read to data1 10 also does reads for 11-13
+            await api.connection.send(Package(0xFFFF, o.address, 3, 10, 0))
         else:
             continue
         await asyncio.sleep(0.5)
