@@ -1,6 +1,7 @@
 import asyncio
 from enum import Enum
 from typing import Callable, List, TYPE_CHECKING, Dict, Any
+import inspect
 
 if TYPE_CHECKING:
     from .api import IngeniumAPI
@@ -34,6 +35,9 @@ class Package:
 
 class IngComponentType(Enum):
     NOT_IMPLEMENTED = -1
+
+    COD_AIR_QUALITY = 8
+
     """
     Actuadores Busing
     """
@@ -79,6 +83,8 @@ class IngComponentType(Enum):
 
     T_KNX_COLOR_PICKER = 9
 
+    def is_air_sensor(self): return self in [self.COD_AIR_QUALITY]
+
     def is_actuator(self): return self in [self.COD2E2S, self.COD4E4S, self.COD6E6S, self.COD_KCTR, self.T_KNX_ONOFF]
 
     def is_meterbus(self): return self in [self.COD_METERBUS]
@@ -141,7 +147,7 @@ class IngComponent:
     output: int
     icon: int
 
-    _notify_update: List[Callable] = []
+    _notify_update: List[Callable]
 
     def __init__(self, json: Dict[str, Any] = None):
         if json is not None:
@@ -149,6 +155,7 @@ class IngComponent:
             self.label = json.get("label")
             self.output = safecast(json.get("output"), int)
             self.icon = safecast(json.get("icon"), int)
+        self._notify_update = []
 
     def __str__(self):
         return "Component ( id='{}', label='{}', output={} )" \
@@ -162,8 +169,10 @@ class IngComponent:
 
     def update_notify(self):
         for cb in self._notify_update:
-            cb()
-
+            try:
+                cb()
+            except BaseException as e:
+                print("Exception {} at {}".format(e, inspect.getsource(cb)))
 
 class IngObject:
     api: 'IngeniumAPI'
@@ -171,7 +180,7 @@ class IngObject:
     address: int
     type: IngComponentType
     type_name: str
-    components: List[IngComponent] = []
+    components: List[IngComponent]
 
     available: bool
 
@@ -219,6 +228,11 @@ class IngActuator(IngObject):
 
         self.consumption = -1
 
+        self.current = -1
+        self.voltage = -1
+        self.active_power_bits = [-1, -1]
+        self.active_power = -1
+
         self.output_state = 0
         self.mode = ACTUATOR_BLIND if self.components[0].icon == 5 else ACTUATOR_NORMAL
 
@@ -345,13 +359,31 @@ class IngActuator(IngObject):
                 self.output_state ^= (1 << p.data2)
                 return self.output_state != old_state
             elif p.data1 == 86:
-                # In case we get a weird zero when it's not a poewr meter
+                # In case we get a weird zero when it's not a power meter
                 if self.consumption == -1 and p.data2 == 0:
                     return False
                 factor = 230 / 10
                 old_cons = self.consumption
                 self.consumption = p.data2 * factor
+                self.current = p.data2 / 10.0
                 return self.consumption != old_cons
+            elif p.data1 == 87:
+                # Voltage
+                old_voltage = self.voltage
+                self.voltage = p.data2
+                return self.voltage != old_voltage
+
+            elif p.data1 == 88:
+                old_value = self.active_power_bits[0]
+                self.active_power_bits[0] = p.data2
+                return self.active_power_bits[0] != old_value
+            
+            elif p.data1 == 89:
+                old_value = self.active_power_bits[1]
+                self.active_power_bits[1] = p.data2
+                if self.active_power_bits[0] != -1:
+                    self.active_power = (self.active_power_bits[0] << 8)  + self.active_power_bits[1]
+                return self.active_power_bits[1] != old_value
 
         return False
 
@@ -525,6 +557,48 @@ class IngSif(IngObject):
         return self.values[mode]
 
 
+class IngAirSensor(IngObject):
+    def __init__(self, api: 'IngeniumAPI', is_knx: bool, address: int, ctype: IngComponentType,
+                 comps: List[IngComponent]):
+        super().__init__(api, is_knx, address, ctype, comps)
+
+        self.bit_values = [0, 0, 0, 0, -1, -1, -1, -1, -1, -1, 0, 0, 0, 0]
+        self.values = [0, 0]
+        self.thresh_values = [0, 0]
+
+    async def update_state(self, p: Package) -> bool:
+        if self.address != p.target:
+            return False
+
+        old_value = self.bit_values[p.data1]
+        self.bit_values[p.data1] = p.data2
+
+        if p.data1 == 3:
+            self.values[0] = (self.bit_values[0] << 8) | self.bit_values[1]
+            self.values[1] = (self.bit_values[2] << 8) | self.bit_values[3]
+            return self.bit_values[3] != old_value
+
+        if p.data1 == 13:
+            self.thresh_values[0] = (self.bit_values[10] << 8) | self.bit_values[11]
+            self.thresh_values[1] = (self.bit_values[12] << 8) | self.bit_values[13]
+            return self.bit_values[13] != old_value
+
+        return False
+
+    def get_available(self, mode: int):
+        # If the current mode value is max, or if _both_ mode values are 0, device is unavailable
+        return self.values[mode] != 65535 and (self.values[0] != 0 or self.values[1] != 0)
+
+    def get_value(self, mode):
+        return self.values[mode]
+
+    def get_threshold_available(self, mode: int):
+        return self.thresh_values[mode] != 65535 and self.thresh_values[mode] != 0
+
+    def get_threshold(self, mode):
+        return self.thresh_values[mode]
+
+
 class IngBusingRegulator(IngObject):
     def __init__(self, api: 'IngeniumAPI', is_knx: bool, address: int, ctype: IngComponentType,
                  comps: List[IngComponent]):
